mod layers;
mod medium;
mod tun_tx_worker;

use crate::tun_tx_worker::TunTxWorker;
use log::LevelFilter;

fn main() -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
    env_logger::builder()
        .filter_level(LevelFilter::Info)
        .parse_default_env()
        .init();

    let mut config = tun::Configuration::default();
    config
        .address((10, 0, 0, 9))
        .netmask((255, 255, 255, 0))
        .destination((10, 0, 0, 1))
        .up();

    #[cfg(target_os = "linux")]
    config.platform_config(|config| {
        // requiring root privilege to acquire complete functions
        config.ensure_root_privileges(true);
    });

    let dev = tun::create(&config)?;

    let (tx, _rx) = dev.split();

    let medium = medium::logger::Logger {};
    let tx_worker = TunTxWorker::new(tx, medium);

    tx_worker.worker_thread();

    Ok(())
}
