use crate::layers::Layer;
use crate::medium::WriteMedium;
use std::io::Read;

pub struct TunTxWorker<T: WriteMedium> {
    layers: Vec<Box<dyn Layer>>,
    tun_reader: tun::Reader,
    tx_medium: T,
}

impl<T: WriteMedium> TunTxWorker<T> {
    pub fn new(tun_reader: tun::Reader, medium: T) -> Self {
        Self {
            layers: vec![],
            tun_reader,
            tx_medium: medium,
        }
    }

    pub fn add_layer(&mut self, layer: Box<dyn Layer>) {
        self.layers.push(layer)
    }

    pub fn worker_thread(mut self) {
        let mut buffer = vec![0; 4096];
        loop {
            buffer.resize(4096, 0);
            let data_read = self.tun_reader.read(&mut buffer).unwrap();

            if data_read == 0 {
                return;
            }

            // TODO: Add layer processing

            self.tx_medium.write_all(&buffer[..data_read]).unwrap();
            buffer.clear();
        }
    }
}
